/* 
 * Eneso GTK+ Decoration Library
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <cairo.h>
#include <gtk/gtk.h>
#include <math.h>
#include "edecor_private.h"
#include "edecor_draw.h"

static PangoLayout*
create_text_layout	(cairo_t* cr, const gchar* text, const gchar* font);


	void 
edecor_draw_text (cairo_t* cr, gint x, gint y, const gchar* text, const gchar* font,
		gboolean center)
{
	PangoLayout* 	layout;
	gint			width, height;

	layout = create_text_layout (cr, text, font);

	/* Move to the position */
	pango_layout_get_size (layout, &width, &height);

	if (center)
	{
		cairo_move_to (cr, x - ((float) width / PANGO_SCALE) / 2,
				y - ((float) height / PANGO_SCALE) / 2);
	}
	else
		cairo_move_to (cr, x, y - ((float) height / PANGO_SCALE) / 2);

	/* Draw */
	pango_cairo_show_layout (cr, layout);

	g_object_unref (layout);
}

	void
edecor_draw_text_unformatted (cairo_t* cr, gint x, gint y, const gchar* text,
		const gchar* font, gboolean center)
{
	PangoLayout*			layout;
	PangoFontDescription*	desc;
	gint					width, height;

	layout = pango_cairo_create_layout (cr);

	/* Use the unparsed text */
	pango_layout_set_text (layout, text, -1);

	/* Load the font */
	desc = pango_font_description_from_string (font);
	pango_layout_set_font_description (layout, desc);
	pango_font_description_free (desc);

	pango_cairo_update_layout (cr, layout);

	/* Move to the position */
	pango_layout_get_size (layout, &width, &height);

	if (center)
	{
		cairo_move_to (cr,
				x - ((float) width / PANGO_SCALE) / 2,
				y - ((float) height / PANGO_SCALE) / 2);
	}
	else
		cairo_move_to (cr, x, y - ((float) height / PANGO_SCALE) / 2);

	/* Draw */
	pango_cairo_show_layout (cr, layout);

	g_object_unref (layout);
}

	void
edecor_draw_text_multiline (cairo_t* cr, gint x, gint y, const gchar* text,
		const gchar* font, gint text_width, gboolean center_y)
{
	PangoLayout*	layout;
	gint			width, height;

	layout = create_text_layout (cr, text, font);

	/* Configure the multiline layout */
	pango_layout_set_width (layout, (float) text_width * PANGO_SCALE);
	pango_layout_set_spacing (layout, 3.0 * PANGO_SCALE);

	/* Move to the position */
	pango_layout_get_size (layout, &width, &height);

	if (center_y)
		cairo_move_to (cr, x, y - ((float) height / PANGO_SCALE) / 2);
	else
		cairo_move_to (cr, x, y);

	/* Draw */
	pango_cairo_show_layout (cr, layout);

	g_object_unref (layout);
}

	void
edecor_draw_round_rect (cairo_t* cr, gint x, gint y, gint width, gint height,
		gint radius)
{
	cairo_move_to (cr, x, y + height - radius);
	cairo_arc (cr, x + radius, y + radius, radius, M_PI, 1.5 * M_PI);
	cairo_arc (cr, x + width - radius, y + radius, radius, 1.5 * M_PI, 2.0 * M_PI);
	cairo_arc (cr, x + width - radius, y + height - radius, radius, 0, 0.5 * M_PI);
	cairo_arc (cr, x + radius, y + height - radius, radius, 0.5 * M_PI, M_PI);
}

	void
edecor_draw_progress_bar (cairo_t* cr, gint x, gint y, gint width, gint height,
		gdouble fraction)
{
	gint	bar_width;

	/* Background */
	cairo_set_source_rgb (cr, 0.8, 0.8, 0.8);
	cairo_rectangle (cr, x, y, width, height);
	cairo_fill (cr);

	/* Bar */
	bar_width = width * fraction;

	cairo_rectangle (cr, x, y, bar_width, height);
	cairo_set_source_rgb (cr, 0.0, 0.65, 0.87);
	cairo_fill (cr);

	/* Border */
	cairo_set_source_rgb (cr, 0.6, 0.6, 0.6);
	cairo_set_line_width (cr, 1.0);
	cairo_rectangle (cr, x + 0.5, y + 0.5, width - 1, height - 1);
	cairo_stroke (cr);
}

	static PangoLayout*
create_text_layout (cairo_t* cr, const gchar* text, const gchar* font)
{
	PangoLayout*			layout;
	PangoFontDescription*	desc;
	PangoAttrList*			attrs;
	gchar*					txt;

	layout = pango_cairo_create_layout (cr);

	/* Parse the text */
	attrs = pango_layout_get_attributes (layout);
	pango_parse_markup (text, -1, 0, &attrs, &txt, NULL, NULL);
	pango_layout_set_attributes (layout, attrs);
	pango_attr_list_unref (attrs);

	/* Use the text without markups */
	pango_layout_set_text (layout, txt, -1);

	/* Load the font */
	desc = pango_font_description_from_string (font);
	pango_layout_set_font_description (layout, desc);
	pango_font_description_free (desc);

	pango_cairo_update_layout (cr, layout);

	return layout;
}
