/* 
 * enPathia
 * Eneso Tecnología de Adaptación, S.L. 
 */

#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <stdio.h>
#include "ehid.h"
#include "calibration.h"
#include "clicker.h"
#include "keyboard.h"
#include "menu.h"
#include "misc.h"
#include "profile.h"
#include "protocol.h"
#include "unlock.h"
#include "window.h"
#include "enpathia.h"

gboolean 	device_connected 	= FALSE;
gboolean 	sensor_connected	= FALSE;

static gint 	current_mode	= 0;
static gboolean stopped			= FALSE;

static gint		menu_disable_count		= 0;
static gint		menu_disable_first_x;
static gboolean	menu_disable_activated	= FALSE;

static void 
handle_raw 			(gchar* message, gint n_bytes);
static void 
handle_stopped 		(gchar* message, gint n_bytes);
static void 
handle_movement 	(gchar* message, gint n_bytes);
static void 
handle_switch 		(gchar* message, gint n_bytes);
static void 
handle_sensor 		(gchar* message, gint n_bytes);

static void 
show_menu_at_position 	(void);
static void 
show_clicker_at_position	(void);
static void 
execute_switch_action 	(gint n);


	void
send_command (gchar command, gchar param_1, gchar param_2)
{
	static gchar buffer [9] = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };

	buffer [0] = command;
	buffer [1] = param_1;
	buffer [2] = param_2;

	ehid_send (buffer);
}

	void
change_mode (gint mode)
{
	/* If we are in the requested mode do nothing */
	if (mode != MODE_NOTIFY && mode == current_mode)
		return;
	
	current_mode = mode;

	/* Send the command and mode-related configuration */
	send_command (COMMAND_SET_MODE, mode, 0);
	send_cfg_speed ();
	send_cfg_time ();
}

	void
handle_message (gchar* message, guint n_bytes)
{
	switch (message [0])
	{
		case NOTIF_RAW: 
			handle_raw (message, n_bytes); 
			break;
		case NOTIF_STOPPED: 
			handle_stopped (message, n_bytes); 
			break;
		case NOTIF_MOVEMENT: 
			handle_movement (message, n_bytes); 
			break;
		case NOTIF_SWITCH:
			handle_switch (message, n_bytes);
			break;
		case NOTIF_SENSOR:
			handle_sensor (message, n_bytes);
			break;
		default: break;
	}
}

	static void
handle_raw (gchar* message, gint n_bytes)
{
	guint	samples [6];
	guint	a, b, c;
	gint	i;

	sensor_connected = TRUE;

	/* Keep only the 8 most significant bits out of the 12 we receive */
	for (i = 0; i < 6; i++) 
		samples [i] = (guint) ((guchar*) message) [i + 2];

	a = samples [0] + (samples [1] << 8);
	b = samples [2] + (samples [3] << 8);
	c = samples [4] + (samples [5] << 8);

	/* Send the samples to the calibration or unlock window */
	if (calibrating)
		calibration_handle_raw_sample (a, b, c);
	else if (unlocking)
		unlock_handle_raw_sample (a, b, c);
}

	static void
handle_stopped (gchar* message, gint n_bytes)
{
	/* If the keyboard, menu or clicker are open they should handle it, otherwise
	 * show the menu */
	if (showing_keyboard && mouse_over_keyboard)
		keyboard_handle_stopped ();
	else if (showing_menu)
		menu_handle_stopped ();
	else if (showing_clicker)
		clicker_handle_stopped ();
	else if (unlocking)
		unlock_handle_stopped ();
	else if (!stopped && !current_profile.config.use_switches && !menu_disabled)
	{
		if (current_profile.config.auto_click)
			show_clicker_at_position ();
		else
			show_menu_at_position ();
	}

	/* Don't show the menu or clicker again until we receive a new movement
	 * notification */
	stopped = TRUE;
}

	static void
handle_movement (gchar* message, gint n_bytes)
{
	gint	x, y;

	if (unlocking)
		unlock_handle_movement (message [1]);
	else if (showing_menu)
		menu_handle_movement (message [1]);
	else if (showing_clicker)
		clicker_handle_movement (message [1]);
	else if (current_profile.config.can_disable_menu && !current_profile.config.use_switches)
	{
		gdk_display_get_pointer (gdk_display_get_default (), NULL, &x, &y, NULL); 

		/* If the pointer is at the screen top, enable or disable the menu */
		if (y == 0)
		{
			if (!menu_disable_activated)
			{
				if (menu_disable_count > 0)
				{
					if (x != menu_disable_first_x)
						menu_disable_count = 0;
					else
					{
						if (menu_disable_count > 5)
						{
							set_menu_disabled (!menu_disabled);

							menu_disable_activated = TRUE;
						}
					}
				}

				menu_disable_first_x = x;
				menu_disable_count++;
			}
		}
		else
		{
			menu_disable_activated = FALSE;
			menu_disable_count = 0;
		}
	}

	stopped = FALSE;
}

	static void
handle_switch (gchar* message, gint n_bytes)
{
	if (!current_profile.config.use_switches)
		return;

	if (showing_keyboard && mouse_over_keyboard)
		keyboard_handle_switch ();
	else
	{
		/* Switch 1 */
		if (message [1] & 0x10)
			execute_switch_action (0);
		else if (message [1] & 0x20)
			execute_switch_action (1);

		/* Switch 2 */
		if (message [1] & 0x01)
			execute_switch_action (2);
		else if (message [1] & 0x02)
			execute_switch_action (3);
	}
}

	static void
handle_sensor (gchar* message, gint n_bytes)
{
	if (sensor_connected)
	{
		sensor_connected = FALSE;

		update_main_window ();
		if (unlocking)
			update_unlock_window ();
	}
}

	static void
show_menu_at_position (void)
{
	gint x, y;

	/* The menu will show up at the pointer position */
	gdk_display_get_pointer (gdk_display_get_default (), NULL, &x, &y, NULL); 

	show_menu (x, y);
}

	static void
show_clicker_at_position (void)
{
	gint x, y;

	gdk_display_get_pointer (gdk_display_get_default (), NULL, &x, &y, NULL); 

	show_clicker (x, y);
}

	static void
execute_switch_action (gint n)
{
	gint action;

	/* Show the menu or execute the action */
	action = current_profile.config.switch_action [n];

	if (action == 0)
		show_menu_at_position ();
	else
		execute_action (action);
}
